/* File: prnclass.c */

/*
 * Copyright 2003, Cypress Semiconductor Corporation
 *
 * This software is owned by Cypress Semiconductor Corporation (Cypress) and
 * is protected by United States copyright laws and international treaty
 * provisions.  Therefore, unless otherwise specified in a separate license
 * agreement, you must treat this software like any other copyrighted
 * material.  Reproduction, modification, translation, compilation, or
 * representation of this software in any other form (e.g., paper, magnetic,
 * optical, silicon, etc.) is prohibited without the express written
 * permission of Cypress.
 *
 * Disclaimer: Cypress makes no warranty of any kind, express or implied, with
 * regard to this material, including, but not limited to, the implied
 * warranties of merchantability and fitness for a particular purpose.
 * Cypress reserves the right to make changes without further notice to the
 * materials described herein. Cypress does not assume any liability arising
 * out of the application or use of any product or circuit described herein.
 * Cypress' products described herein are not authorized for use as
 * components in life-support devices.
 *
 * This software is protected by and subject to worldwide patent coverage,
 * including U.S. and foreign patents. Use may be limited by and subject to
 * the Cypress Software License Agreement.
 */


/* Included files. */

#include <string.h>

#include "fwx.h"

#ifdef FWX_INCLUDE_HOST_USB_SUPPORT

#include "td.h"
#include "prnclass.h"
#include "printbuffer.h"
#include "board.h"
#include "utils.h"
#include "debug.h"
#include "logger.h"



/* Macro definitions. */

#undef  USE_SET_INTERFACE



/* Data. */

static bool             driver_in_use = FALSE;
static bool             enumerated = FALSE;

static bool             print_started = FALSE;

static USB_DEVICE       *g_dev = 0;

static uint16           packet_byte_count;

static bool             dma_is_armed = FALSE;

static bool             in_cmd_phase = FALSE;

static bool             buffer_ready = FALSE;

static uint16           num_urbs_submitted = 0;
static uint16           num_urbs_done = 0;

static bool             using_bidi = FALSE;

#ifdef USE_SET_INTERFACE
static uint16          interface_number = 0;
#endif /* USE_SET_INTERFACE */

static uint16          bulk_out_endpoint = 0;
static uint16          bulk_in_endpoint = 0;
static uint16          int_in_endpoint = 0;

static uint16          printbuffer_size = 0;


/* Function declarations. */

static bool has_endpoint( USB_ENDPOINT_DESCRIPTOR *ep, uint16 xfer_type,
                          uint16 direction, uint16 *endpoint );

static void handle_printbuffer_done( URB *urb );

/* static void get_status_done( URB *urb ); */

                         

/* Driver interface routines. */

static uint16 prnclass_start_uni( USB_DEVICE *dev );
static uint16 prnclass_start_bidi( USB_DEVICE *dev );
static uint16 prnclass_start( USB_DEVICE *dev );
static uint16 prnclass_stop( USB_DEVICE *dev );
static void   prnclass_run( USB_DEVICE *dev );
static uint16 prnclass_ioctl(USB_DEVICE *dev,uint16 cmd,uint16 d1,uint16 d2);



/* Data. */

CLASS_DRIVER printer_class_driver_uni =
{
    0,                      /* uint8   class;                       */
    0,                      /* uint8   subclass;                    */
    USB_PRINTER_CLASS,      /* uint8   if_class                     */
    USB_PRINTER_SUBCLASS,   /* uint8   if_subclass.                 */
    1,                      /* uint8   protocol;                    */
    0x0000,                 /* uint16  vendor_ID;                   */
    0x0000,                 /* uint16  product_ID;                  */
    prnclass_start_uni,     /* uint16  (*start)( USB_DEVICE *dev ); */
    prnclass_stop,          /* uint16  (*stop)(void);               */
    prnclass_run,           /* void    (*run)(void);                */
    prnclass_ioctl,
    PRNCLASS_DEV_ID         /* uint16  id                           */
};



CLASS_DRIVER printer_class_driver_bidi =
{
    0,                      /* uint8   class;                       */
    0,                      /* uint8   subclass;                    */
    USB_PRINTER_CLASS,      /* uint8   if_class                     */
    USB_PRINTER_SUBCLASS,   /* uint8   if_subclass.                 */
    2,                      /* uint8   protocol;                    */
    0x0000,                 /* uint16  vendor_ID;                   */
    0x0000,                 /* uint16  product_ID;                  */
    prnclass_start_bidi,    /* uint16  (*start)( USB_DEVICE *dev ); */
    prnclass_stop,          /* uint16  (*stop)(void);               */
    prnclass_run,           /* void    (*run)(void);                */
    prnclass_ioctl,
    PRNCLASS_DEV_ID         /* uint16  id                           */
};




/* Implementation. */


void hpi_tx_empty_isr(void)
{
    /* TODO: Fill in your ISR functionality here. */
}



void hpi_rx_full_isr(void)
{
    /* TODO: Fill in your ISR functionality here. */

    printbuffer_size = READ_REGISTER(0xC0C6);

    tpl_ioctl( PRNCLASS_DEV_ID, PRNCLASS_IOCTL_SEND_TEST_PAGE, 0, 0 );
}



static bool has_endpoint( USB_ENDPOINT_DESCRIPTOR *ep, uint16 xfer_type,
                          uint16 direction, uint16 *endpoint )
{
    bool    result = FALSE;
    
    while( ep->bDescriptorType == USB_ENDPOINT_DESCRIPTOR_TYPE )
    {
        if( ( ep->bmAttributes & 3 ) == xfer_type )
        {
            if( ( ep->bEndpointAddress & 0x80 ) == direction )
            {
                *endpoint = ( ep->bEndpointAddress & 3 );
            
                result = TRUE;
            }
        }
        
        ++ep;
    }
    
    return result;
}




void prnclass_specify_interface( USB_INTERFACE_DESCRIPTOR *ip,
                                 USB_ENDPOINT_DESCRIPTOR *ep )
{
    uint16  endpoint;

#ifdef USE_SET_INTERFACE    
    interface_number = ip->bInterfaceNumber;
#endif /* USE_SET_INTERFACE */
    
    if( has_endpoint( ep, USB_BULK_TRANSFER_TYPE, USB_OUT_ENDPOINT, &endpoint ) )
    {
        bulk_out_endpoint = endpoint;
    }
    else
    {
        bulk_out_endpoint = 0;
    }
    
    if( has_endpoint( ep, USB_BULK_TRANSFER_TYPE, USB_IN_ENDPOINT,  &endpoint ) )
    {
        bulk_in_endpoint = endpoint;
    }
    else
    {
        bulk_in_endpoint = 0;
    }
    
    if( has_endpoint( ep, USB_INTERRUPT_TRANSFER_TYPE, USB_IN_ENDPOINT, &endpoint ) )
    {
        int_in_endpoint = endpoint;
    }
    else
    {
        int_in_endpoint = 0;
    }
}






bool prnclass_use_this_intf_descr( USB_INTERFACE_DESCRIPTOR *ip )
{
    bool    result = FALSE;
    bool    has_bulk_out = FALSE;
    bool    has_bulk_in = FALSE;
    bool    has_int_in = FALSE;
    
    if( ip->bInterfaceClass == USB_PRINTER_CLASS
        && ip->bInterfaceSubClass == USB_PRINTER_SUBCLASS )
    {
        USB_ENDPOINT_DESCRIPTOR *ep = (USB_ENDPOINT_DESCRIPTOR *) ( ip + 1 );
        uint16  endpoint;

        /* We will use this interface if it has bulk-out and does
         * not have interrupt-in.
         */
    
        has_bulk_out = has_endpoint( ep, USB_BULK_TRANSFER_TYPE, USB_OUT_ENDPOINT, &endpoint );
        has_bulk_in  = has_endpoint( ep, USB_BULK_TRANSFER_TYPE, USB_IN_ENDPOINT,  &endpoint );
        has_int_in   = has_endpoint( ep, USB_INTERRUPT_TRANSFER_TYPE, USB_IN_ENDPOINT, &endpoint );
    }
    
    if( has_bulk_out == TRUE && has_int_in == FALSE )
    {
        result = TRUE;
    }
    
    return result;
}






#ifdef USE_SET_INTERFACE

static void set_interface_handler( URB *urb )
{
    if( urb->status == 0 )
    {
        enumerated = TRUE;
    }
    
    /* Release the URB that we just handled. */
    
    release_URB( urb );
}

#endif /* USE_SET_INTERFACE */





void send_printbuffer( USB_DEVICE *dev )
{
    if( enumerated && print_started == FALSE )
    {
        URB     *urb = 0;
    
        print_started = TRUE;
        
	/*
	urb = send_request(dev, EP0, TD_CTRL_DIR_IN, 0xA1, 1, 0, 0, 1, get_status_done);

	if( urb == 0 )
	{
	    WRITE_REGISTER(0xC0C6, 0x1234);
	}
	*/

        urb = alloc_URB( 0, 0 );

        if( urb != 0 )
        { 
            urb->transfer_buffer = printbuffer;

            urb->dev = dev;
            urb->dir = TD_CTRL_DIR_OUT;
            urb->usb_dev_addr = dev->address;
            urb->endpoint = bulk_out_endpoint;
            urb->speed = dev->speed;
            urb->type = USB_BULK_TRANSFER_TYPE;
            urb->buffer_length = printbuffer_size;  
            urb->callback = handle_printbuffer_done;

            if( td_submit_URB( urb ) != SUCCESS )
	      {
                release_URB( urb );

                urb = 0;
            }
        }
    }
}








static void handle_printbuffer_done( URB *urb )
{
    print_started = FALSE;

    WRITE_REGISTER(0xC0C6, printbuffer_size);
    
    release_URB( urb );
}


/*
static void get_status_done( URB *urb )
{

    print_started = FALSE;

    WRITE_REGISTER(0xC0C6, *((char *) urb->transfer_buffer));
    
    release_URB( urb );
}
*/

/* Driver functionality. */

static uint16 prnclass_start_uni( USB_DEVICE *dev )
{
    return prnclass_start( dev );
}


static uint16 prnclass_start_bidi( USB_DEVICE *dev )
{
    using_bidi = TRUE;
    
    return prnclass_start( dev );
}





static uint16 prnclass_start( USB_DEVICE *dev )
{
    uint16  result = ERROR;
    
    DBGLOG( TRC_driver_start, 0 );

    if( driver_in_use == FALSE )
    {
        print_started = FALSE;
        
        dma_is_armed = FALSE;
        
        g_dev = dev;
        
        enumerated = TRUE;

        num_urbs_submitted = 0;
        num_urbs_done = 0;

        result = SUCCESS;

        DBGLOG( TRC_driver_start, 0x8001 );
        
        in_cmd_phase = TRUE;
        
    }

    DBGLOG( TRC_driver_start, TRC_exit );
    
    return result;
}





static uint16 prnclass_stop( USB_DEVICE *dev )
{
    DBGLOG( TRC_driver_stop, 0 );
    
    if( driver_in_use == TRUE )
    {
        enumerated = FALSE;
        
        using_bidi = FALSE;
        
#ifdef USE_SET_INTERFACE
        interface_number = 0;
#endif /* USE_SET_INTERFACE */

        bulk_out_endpoint = 0;
        bulk_in_endpoint = 0;
        int_in_endpoint = 0;

        driver_in_use = FALSE;
    }
    
    DBGLOG( TRC_driver_stop, TRC_exit );

    return SUCCESS;
}



/* This is the driver's idle function. */

static void prnclass_run( USB_DEVICE *dev )
{
}



static uint16 prnclass_ioctl( USB_DEVICE *dev, uint16 cmd,
                              uint16 d1, uint16 d2 )
{
    switch( cmd )
    {
    case PRNCLASS_IOCTL_SEND_TEST_PAGE:
      send_printbuffer( dev );
        break;
    }

    return SUCCESS;
}



#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */




/* End of file: prnclass.c */

