/* File: sie1.c */

/*
 * Copyright 2003, Cypress Semiconductor Corporation
 *
 * This software is owned by Cypress Semiconductor Corporation (Cypress) and
 * is protected by United States copyright laws and international treaty
 * provisions.  Therefore, unless otherwise specified in a separate license
 * agreement, you must treat this software like any other copyrighted
 * material.  Reproduction, modification, translation, compilation, or
 * representation of this software in any other form (e.g., paper, magnetic,
 * optical, silicon, etc.) is prohibited without the express written
 * permission of Cypress.
 *
 * Disclaimer: Cypress makes no warranty of any kind, express or implied, with
 * regard to this material, including, but not limited to, the implied
 * warranties of merchantability and fitness for a particular purpose.
 * Cypress reserves the right to make changes without further notice to the
 * materials described herein. Cypress does not assume any liability arising
 * out of the application or use of any product or circuit described herein.
 * Cypress' products described herein are not authorized for use as
 * components in life-support devices.
 *
 * This software is protected by and subject to worldwide patent coverage,
 * including U.S. and foreign patents. Use may be limited by and subject to
 * the Cypress Software License Agreement.
 */


/* 
 * Design Example 3 is a device to host keyboard passthrough. 
 *
 * SIE1 is an USB Host, only a HID boot keyboard (no hub) is allowed to be attached.
 *
 * The keyboard must be plugged in first before the PC Host is. The 
 * "keyboard" won't initialize/enumerate until the descriptors are read from
 * the actual keyboard.
 *
 * If the passthrough logic works well, a mouse should work also. With
 * some minor changes any HID device can work.
 *
 */

/* Included files. */

#include <string.h>

#include "fwx.h"
#include "usb.h"
#include "utils.h"
#include "board.h"
#include "app.h"
#include "prnclass.h"
#include "sie1.h"
#include "urb.h"
#include "usb.h"
#include "td.h"
#include "host.h"
#include "debug.h"
#include "device.h"
#include "logger.h"
#include "printbuffer.h"
      

/* Macro and constant definitions. */

#define NUM_SUPPORTED_PORTS                  2                            



#ifdef FWX_INCLUDE_HOST_USB_SUPPORT

/* port_change[0] is for the A port.
 * port_change[1] is for the B port.
 */
static bool port_change[NUM_SUPPORTED_PORTS] = { FALSE, FALSE };

static bool direct_connect_present[NUM_SUPPORTED_PORTS] = { FALSE };

#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */


/* Function declarations. */

static void sie1_enumeration_notify( USB_DEVICE *dev );


#ifdef FWX_INCLUDE_HOST_USB_SUPPORT
/* For managing USB_DEVICE storage within the device_map strucutre. */

#undef SIE1_INCLUDE_FIND_DEVICE


#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */


void sie1_init(void)
{
    device_map_init();
    
    sie1_init_host();
}



/* go find the attached HID device */

void sie1_init_host(void)
{
    DBGLOG( TRC_sie1_init_host, 0 );
    
    WRITE_REGISTER( HUSB_pEOT_ADR, 1200 );

    husb1_init();
    
    INPLACE_OR( HOST1_IRQ_EN_REG, (A_CHG_IRQ_EN | B_CHG_IRQ_EN) );

    DBGLOG( TRC_sie1_init_host, READ_REGISTER( HOST1_IRQ_EN_REG ) );
}


#ifdef FWX_INCLUDE_HOST_USB_SUPPORT
void sie1_check_for_connected_devices(void)
{
    uint16      port;
    uint16      reg;

    
    /* Disable the insert/remove interrupts. */
    
    INPLACE_AND( DEV1_IRQ_EN_REG, ~(A_CHG_IRQ_EN | B_CHG_IRQ_EN) );


    /* Check for connected devices. */

    for( port = 0; port < 2; ++port )
    {
        reg = husb_reset( 20, port );

        if( (reg & 2) == 0 )
        {
            enumerate_device( SIE1, port,
                              &direct_connect_present[port],
                              sie1_enumeration_notify );
        }
    }
    
    /* Prior to re-enabling the interrupts, make sure the insert/remove interrupt is cleared. */
    
    INPLACE_OR( HOST1_STAT_REG, A_CHG_IRQ_EN | B_CHG_IRQ_EN );
    
    /* Enable the insert/remove interrupts. */
    
    INPLACE_OR( DEV1_IRQ_EN_REG, A_CHG_IRQ_EN | B_CHG_IRQ_EN ); 


    /* Make sure that our port_change variables are clear. */
    
    port_change[0] = 0;
    port_change[1] = 0;
}
#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */


     
#ifdef FWX_INCLUDE_SIE1_HOST_DONE_ISR
void sie1_host_done_isr(void)
{
    /* TODO: Fill in your ISR functionality here. */
}
#endif /* FWX_INCLUDE_SIE1_HOST_DONE_ISR */



#ifdef FWX_INCLUDE_SIE1_HOST_SOF_ISR
void sie1_host_sof_isr(void)
{                                                                                   
    /* TODO: Fill in your ISR functionality here. */
}
#endif /* FWX_INCLUDE_SIE1_HOST_SOF_ISR */



#ifdef FWX_INCLUDE_SIE1_HOST_INS_REM_ISR
#ifdef FWX_INCLUDE_HOST_USB_SUPPORT
void sie1_host_ins_rem_isr( uint16 port_status )
{
    
     /* port_status contains the original contents of
     * the HOST1_STATUS_REGISTER.
     * The lower level assembly has already cleared
     * the bits in the interrupt status register.
     */

    DBGLOG( TRC_host_ins_rem, port_status );

    if( ( port_status & A_CHG_IRQ_FLG ) != 0 )
    {
        DBGLOG( TRC_host_ins_rem, 0x8000 );
        port_change[0] = TRUE;
    }

    if( ( port_status & B_CHG_IRQ_FLG ) != 0 )
    {
        DBGLOG( TRC_host_ins_rem, 0x8001 );
        port_change[1] = TRUE;
    }
    
}
#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */
#endif /* FWX_INCLUDE_SIE1_HOST_INS_REM_ISR */




/* This function is called by enumerate_device() when device
 * enumeration is complete or has failed.
 */
 
static void sie1_enumeration_notify( USB_DEVICE *dev )
{
    switch( dev->enum_state )
    {
    case ES_COMPLETE:
    
        if( dev->direct_connect == TRUE )
        {
            DBGLOG( TRC_enum_notify, 1 );
            direct_connect_present[dev->port] = TRUE;

	    if( dev->inf_descr.bInterfaceClass == USB_PRINTER_CLASS )
	    {
	        WRITE_REGISTER(0xC0C6, printbuffer);
	    }
        }
        else
        {
            /* If you support hubs. */
            
            DBGLOG( TRC_enum_notify, 3 );
        }
        break;

    case ES_NO_DEVICE_CONNECTED:
        
        direct_connect_present[dev->port] = FALSE;
        
        tpl_unlink_all_port( dev->port );
        
        device_cleanup( dev->sie, dev->port );
        
        dealloc_device( dev );
        
        DBGLOG( TRC_enum_notify, 4 );
        DBGLOG( TRC_enum_notify, dev->enum_state );
        break;
        
    case ES_ENUMERATION_FAILED:
        direct_connect_present[dev->port] = FALSE;
        
        tpl_unlink_all_port( dev->port );
     
        device_cleanup( dev->sie, dev->port );
        dealloc_device( dev );
     
        DBGLOG( TRC_enum_notify, 0xee01 );
        DBGLOG( TRC_enum_notify, dev->enum_state );
        break;
        
    default:
        SYS_ERROR( "unhandled enumeration state" );
     
        direct_connect_present[dev->port] = FALSE;
     
        tpl_unlink_all_port( dev->port );
     
        device_cleanup( dev->sie, dev->port );
        dealloc_device( dev );
      
        DBGLOG( TRC_enum_notify, 0xee02 );
        DBGLOG( TRC_enum_notify, dev->enum_state );
        break;
    }
}




void sie1_idle(void)
{
#ifdef FWX_INCLUDE_HOST_USB_SUPPORT
    int         port;

    /* check for a device removal or insertion */
    
    for( port = 0; port < 2; ++port )
    {
        if( port_change[port] == TRUE )
        {
            port_change[port] = FALSE;
            
            enumerate_device( SIE1, port,
                              &direct_connect_present[port],
                              sie1_enumeration_notify );
        }
    }
#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */
}



#ifdef FWX_INCLUDE_HOST_USB_SUPPORT
void sie1_host_cleanup(void)
{
    DBGLOG( TRC_sie1_host_clean, 0 );
    
    /* Stop all TD processing. */
    
#ifdef FWX_INCLUDE_TD_SUPPORT
    /* Clear the TD list pointer. */
    WRITE_REGISTER( HUSB_SIE1_pCurrentTDPtr, 0 );
    td_stop( SIE1 );

#endif /* FWX_INCLUDE_TD_SUPPORT */


#ifdef FWX_INCLUDE_TPL_SUPPORT
    /* Stop all drivers */
    tpl_stop_all();

    /* Remove all drivers from active service. */
    tpl_unlink_all();
#endif /* FWX_INCLUDE_TPL_SUPPORT */

    td_clear_sie( SIE1, 0 );

#ifdef FWX_INCLUDE_SIE2
    td_clear_sie( SIE1, 1 );
#endif /* FWX_INCLUDE_SIE2 */

    dealloc_all_devices();
    
    direct_connect_present[0] = FALSE;
    direct_connect_present[1] = FALSE;
}
#endif /* FWX_INCLUDE_HOST_USB_SUPPORT */




void set_port_change( uint16 port )
{
    port_change[port] = TRUE;
}





/* End of file: sie1.c */
